import requests
from bs4 import BeautifulSoup
from urllib.parse import urlparse, parse_qs
import threading

from page import Page, PageSequence, SearchResults

# TODO: Use logging module instead.
DEBUG = False

if DEBUG:
    from pprint import pprint

_BILL_SEARCH_URI = "https://capitol.texas.gov/Search/BillSearch.aspx"
_VIEWSTATE_ID = '__VIEWSTATE'
_PREVPAGE_ID = '__PREVIOUSPAGE'
_BILL_SEARCH_POSTBACK_PARAMS = {
    "__EVENTTARGET": "",
    "__EVENTARGUMENT": "",
    "__LASTFOCUS": "",
    ## "__VIEWSTATE": extracted from cold response
    ## "__PREVIOUSPAGE": extracted from cold response
    "cboLegSess": "86R",
    "chkHouse": "on",
    "chkSenate": "on",
    "chkB": "on",
    "chkJR": "on",
    "btnSearch": "Search",
    "usrLegislatorsFolder$cboAuthor": "",
    "usrLegislatorsFolder$chkPrimaryAuthor": "on",
    "usrLegislatorsFolder$authspon": "rdoOr",
    "usrLegislatorsFolder$cboSponsor": "",
    "usrLegislatorsFolder$chkPrimarySponsor": "on",
    "usrSubjectsFolder$subjectandor": "rdoOr",
    "usrSubjectsFolder$txtCodes": "",
    "usrCommitteesFolder$cboCommittee": "",
    "usrCommitteesFolder$status": "rdoStatusBoth",
    "usrActionsFolder$actionandor": "rdoOr",
    "usrActionsFolder$txtCodes": "",
    "usrActionsFolder$lastaction": "rdoLastActionNo",
    "usrActionsFolder$dtActionOnDate": "",
    "usrActionsFolder$dtActionFromDate": "",
    "usrActionsFolder$dtActionToDate": "",
    }


def _hidden_input_value(soup, id):
    return soup \
        .find(name='input', attrs={'type':'hidden','id':id}) \
        .attrs['value']


def _search_id(bill_search_redirect_uri):
    '''
    Returns only the ID, given a redirect URI from BillSearch.aspx like

        /Search/BillSearchResults.aspx?NSP=1&SPL=False& ... &ID=BMI3UVlA2

    '''
    parsed = urlparse(bill_search_redirect_uri)
    query_params = parse_qs(parsed.query)
    id_values = query_params['ID']
    return id_values[0]


def _postback_data(cold_response):
    ## Prepare the POSTBACK to BillSearch.aspx.
    soup = BeautifulSoup(cold_response.text, 'html.parser')
    post_data = _BILL_SEARCH_POSTBACK_PARAMS.copy()
    post_data[_VIEWSTATE_ID] = _hidden_input_value(soup, _VIEWSTATE_ID)
    post_data[_PREVPAGE_ID]  = _hidden_input_value(soup, _PREVPAGE_ID)
    
    if DEBUG: pprint(post_data)
    
    return post_data


def _new_search_id(session):
    cold_response = session.get(_BILL_SEARCH_URI)
    postback_response = session.post(
        _BILL_SEARCH_URI, 
        data=_postback_data(cold_response), 
        allow_redirects=False) # <== This parameter is critical!
    redirect_uri = postback_response.headers.get('Location', None)

    if postback_response.status_code != 302 or redirect_uri is None:
        raise RuntimeError("Failed to trick BillSearch.aspx into issuing a fresh search ID.")

    id = _search_id(redirect_uri)
    return id


def _http_get_factory(requests_session):
    def http_get(uri):
        # TODO: BUG: Distributed Computing Fallacy #1: The network is reliable.
        response = requests_session.get(uri, allow_redirects=False)

        if response.status_code != 200:
            raise RuntimeError(
                'Unexpected response HTTP status {} while fetching {}'.format(
                    response.status_code,
                    uri))

        return response.text
    
    return http_get


def search(search_results_uri_without_id, search_id=None, requests_session=None):
    '''
    Repeats a Texas Legislature Bill Search and returns a tuple of 
    (search_id, search_results).

        search_results_uri_without_id - the absolute URI of your BillSearchResults.aspx 
            page WITHOUT THE "ID=abc123DEF" QUERY PARAMETER!  E.g., 
            "https://capitol.texas.gov/Search/BillSearchResults.aspx?NSP=1&SPL=False&...&TT="

        search_id (OPTIONAL) - the ID of a previous search you conducted recently 
            (past day-ish?) if available.  If you omit this parameter, a new one 
            will be fetched automatically.

        requests_session (OPTIONAL) - a Requests library Session object, if you
            are using one.  If you omit this parameter, a new Session will be
            created automatically.
    '''
    session = requests_session if requests_session else requests.Session()
    http_get = _http_get_factory(session)

    id = search_id if search_id else _new_search_id(session)

    # TODO: strip out and replace the ID query parameter if it was included
    results_uri = search_results_uri_without_id + '&ID=' + id
    first_page = Page(http_get(results_uri), results_uri)
    page_seq = PageSequence(http_get, first_page)
    search_results = SearchResults(page_seq)
    return id, search_results


if __name__ == '__main__':
    # NOTICE: We chopped off the 'ID=cMVddWbvD' query param!
    SESSION_86_BILLS = 'https://capitol.texas.gov/Search/BillSearchResults.aspx?NSP=1&SPL=False&SPC=False&SPA=True&SPS=False&Leg=86&Sess=R&ChamberH=True&ChamberS=True&BillType=B;JR;CR;R;;;&AuthorCode=&SponsorCode=&ASAndOr=O&IsPA=True&IsJA=False&IsCA=False&IsPS=True&IsJS=False&IsCS=False&CmteCode=&CmteStatus=&OnDate=&FromDate=&ToDate=&FromTime=&ToTime=&LastAction=False&Actions=S000;S001;H001;&AAO=O&Subjects=&SAO=&TT=' # NO ID!

    id, search_results = search(SESSION_86_BILLS)

    # This is just a very simple demonstration that we can actually get 
    # search results directly from BillSearchResults.aspx.
    print('{} bills found...'.format(search_results.count))
    for index, bill in enumerate(search_results.bills):
        if index > 40:
            break

        print('result {} of {}: {}'.format(
            index+1, 
            search_results.count, 
            bill))


## TODO: encapsulate most of this module into a stateful class



from page import _parse, _nearest_ancestor_table, Page, Result, PageSequence
import unittest
import datetime


BILL_SEARCH_RESULTS_ABS_URI = 'https://capitol.texas.gov/Search/BillSearchResults.aspx'

FULL_RESULT_PAGE_FILENAME = 'Test/BillSearchResults.aspx.FullPage.html'
with open(FULL_RESULT_PAGE_FILENAME, 'r') as f:
    FULL_RESULT_PAGE_HTML = f.read()

RESULT_LAST_PAGE_FILENAME = 'Test/BillSearchResults.aspx.LastPage.html'
with open(RESULT_LAST_PAGE_FILENAME, 'r') as f:
    RESULT_LAST_PAGE_HTML = f.read()

NO_RESULTS_PAGE_FILENAME = 'Test/BillSearchResults.aspx.NoMatches.html'
with open(NO_RESULTS_PAGE_FILENAME, 'r') as f:
    NO_RESULTS_PAGE_HTML = f.read()


class TestNearestAncestorTable(unittest.TestCase):
    markup = '''
        <html>
            <body>
                <table id="outer">
                    <tr>
                        <td id="outer_td">
                            <table id="inner">
                                <tr>
                                    <td id="inner_td"></td>
                                </tr>
                            </table>
                        </td>
                    </tr>
                </table>
            </body>
        <html>
        '''

    @classmethod
    def setUpClass(cls):
        cls.soup = _parse(cls.markup)

    def test_element_supplied_is_table(self):
        # Arrange
        soup = TestNearestAncestorTable.soup
        inner_table = soup.find(id='inner')

        # Act
        actual = _nearest_ancestor_table(inner_table)

        # Assert
        self.assertIs(actual, soup.find(id='outer'))


    def test_normal_case(self):
        # Arrange
        soup = TestNearestAncestorTable.soup
        outer_td = soup.find(id='outer_td')

        # Act
        actual = _nearest_ancestor_table(outer_td)

        # Assert
        self.assertIs(actual, soup.find(id='outer'))        

        
    def test_no_ancestor_is_table(self):
        # Arrange
        soup = TestNearestAncestorTable.soup
        body = soup.find(name='body')

        # Act & Assert
        with self.assertRaises(ValueError):
            _nearest_ancestor_table(body)


class TestResult(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        TestResult.soup = _parse(FULL_RESULT_PAGE_HTML)
        first_txicon = TestResult.soup.find(name='img', attrs={'src':'../Images/txicon.gif'})
        assert first_txicon is not None
        TestResult.result_table = _nearest_ancestor_table(first_txicon)

    def test_title(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)
        self.assertEqual(actual.title, 'HB 21')

    def test_history_uri(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)
        self.assertEqual(
            actual.history_uri,
            'https://capitol.texas.gov/BillLookup/History.aspx?LegSess=86R&Bill=HB21')

    def test_author_pattern(self):
        '''
        Ensure that the author regular expression handles whitespace and
        punctuation in the author's name(s).
        '''
        input = ''':
					 Romero, Jr. | et al.
            '''
        m = Result._AUTHOR_PATTERN.match(input)
        self.assertEqual(m.group(1), 'Romero, Jr. | et al.')

    def test_author(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)
        self.assertEqual(actual.author, 'Canales')

    def test_caption_version(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)

        tds = [ td for td in TestResult.result_table.find_all('td') ]

        self.assertEqual(actual.caption_version, 'Introduced')

    def test_caption(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)
        self.assertEqual(
            actual.caption,
            'Relating to exempting textbooks purchased, used, or consumed by university and college students from the sales and use tax for limited periods.')

    def test_last_action(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)
        self.assertEqual(actual.last_action, 'H Filed')

    def test_last_action_date(self):
        actual = Result(TestResult.result_table, BILL_SEARCH_RESULTS_ABS_URI)
        self.assertEqual(actual.last_action_date, datetime.date(2018, 11, 12))


class TestPage(unittest.TestCase):

    def test_total_result_count(self):
        # Act
        actual = Page(FULL_RESULT_PAGE_HTML, BILL_SEARCH_RESULTS_ABS_URI)

        # Assert
        self.assertEqual(actual.total_result_count, 1140)

    def test_next_page_uri(self):
        # Arrange
        expected = 'https://capitol.texas.gov/Search/BillSearchResults.aspx?CP=2&shCmte=False&shComp=False&shSumm=False&NSP=1&SPL=False&SPC=False&SPA=True&SPS=False&Leg=86&Sess=R&ChamberH=True&ChamberS=True&BillType=B;JR;;;;;&AuthorCode=&SponsorCode=&ASAndOr=O&IsPA=True&IsJA=False&IsCA=False&IsPS=True&IsJS=False&IsCS=False&CmteCode=&CmteStatus=&OnDate=&FromDate=&ToDate=&FromTime=&ToTime=&LastAction=False&Actions=S000;S001;H001;&AAO=O&Subjects=&SAO=&TT=&ID=jNkeLN5Sp'

        # Act
        actual = Page(FULL_RESULT_PAGE_HTML, BILL_SEARCH_RESULTS_ABS_URI)

        # Assert
        self.assertEqual(actual.next_page_uri, expected)

    def test_next_page_uri_last_page(self):
        # Act 
        last_page = Page(RESULT_LAST_PAGE_HTML, BILL_SEARCH_RESULTS_ABS_URI)
        actual = last_page.next_page_uri

        # Assert
        self.assertIsNone(actual)

    def test_results(self):
        # Act
        actual = Page(FULL_RESULT_PAGE_HTML, BILL_SEARCH_RESULTS_ABS_URI)

        # Assert
        self.assertEqual(len(actual.results), 25)
        self.assertEqual(actual.results[0].title, 'HB 21')
        self.assertEqual(actual.results[-1].title, 'HB 45')

    def test_no_results(self):
        # Act 
        actual = Page(NO_RESULTS_PAGE_HTML, BILL_SEARCH_RESULTS_ABS_URI)

        # Assert
        self.assertEqual(actual.total_result_count, 0)
        self.assertIsNone(actual.next_page_uri)
        self.assertEqual(actual.results, [])


class TestPagedResults(unittest.TestCase):
    def test_multiple_pages(self):
        # Arrange
        first_page_html = FULL_RESULT_PAGE_HTML
        last_page_html = RESULT_LAST_PAGE_HTML

        first_page = Page(first_page_html, BILL_SEARCH_RESULTS_ABS_URI)
        last_page = Page(last_page_html, BILL_SEARCH_RESULTS_ABS_URI)
        fake_http_get_call_count = 0

        def fake_http_get(uri):
            nonlocal fake_http_get_call_count
            fake_http_get_call_count += 1
            return last_page_html

        # Act
        page_seq = PageSequence(fake_http_get, first_page)
        pages = list( p for p in page_seq.pages )

        # Assert
        self.assertEqual(pages, [ first_page, last_page ])
        self.assertEqual(fake_http_get_call_count, 1)

    def test_single_page(self):
        # Arrange
        only_page_html = RESULT_LAST_PAGE_HTML
        only_page = Page(only_page_html, BILL_SEARCH_RESULTS_ABS_URI)

        def fake_http_get(uri):
            raise RuntimeError('This should never be called!')

        # Act
        page_seq = PageSequence(fake_http_get, only_page)
        actual = list( p for p in page_seq.pages )

        # Assert
        self.assertEqual(actual, [ only_page, ])

    def test_does_NOT_load_same_page_twice(self):
        # Arrange
        first_page_html = FULL_RESULT_PAGE_HTML
        last_page_html = RESULT_LAST_PAGE_HTML

        first_page = Page(first_page_html, BILL_SEARCH_RESULTS_ABS_URI)
        last_page = Page(last_page_html, BILL_SEARCH_RESULTS_ABS_URI)
        fake_http_get_call_count = 0

        def fake_http_get(uri):
            nonlocal fake_http_get_call_count
            fake_http_get_call_count += 1
            return last_page_html

        # Act
        page_seq = PageSequence(fake_http_get, first_page)

        # iterate through the pages twice (but we should only 
        # see one load operation, and the resulting pages should 
        # be the same instances)
        pages1 = list( p for p in page_seq.pages )
        pages2 = list( p for p in page_seq.pages )

        # Assert
        self.assertEqual(pages1, [ first_page, last_page ])
        self.assertEqual(pages2, pages1)
        self.assertIs(pages1[0], pages2[0])
        self.assertIs(pages1[1], pages2[1])
        self.assertEqual(fake_http_get_call_count, 1)


class TestSearchResults(unittest.TestCase):

    class Generator(object):
        def __init__(self, start, stop):
            self.start = start
            self.stop = stop

        def items(self):
            for i in range(self.start, self.stop):
                yield i

    class NestedGenerator(object):
        def __init__(self, *args):
            self.gens = args

        def items(self):
            for g in self.gens:
                yield from g.items()

    def test_nested_generator(self):
        # Arrange
        gen10 = TestSearchResults.Generator(0, 10)
        gen20 = TestSearchResults.Generator(10, 20)
        nestedgen = TestSearchResults.NestedGenerator(gen10, gen20)

        # Act
        actual = [ i for i in nestedgen.items() ]

        # Assert
        expected = [ x for x in range(20) ]
        self.assertEqual(actual, expected)


if __name__ == '__main__':
    unittest.main()



from bs4 import BeautifulSoup
import re
import datetime
import threading
import urllib.parse


DEBUG = False


def _parse(html):
    '''
    Internally and in test, prefer this method to calling 
    the BeautifulSoup constructor directly, so that the
    parser can be changed out later, if desired.
    '''
    return BeautifulSoup(html, 'html.parser')


def _nearest_ancestor_table(element):
    '''
    Given a BeautifulSoup element, returns the nearest ancestor 
    <table> of that element, not including the supplied element 
    itself.
    '''
    for ancestor in element.parents:
        if ancestor.name == 'table':
            return ancestor
    
    raise ValueError('Element has no <table> ancestor!')


class SearchResults(object):
    def __init__(self, page_sequence):
        self.page_seq = page_sequence

    @property
    def count(self):
        return self.page_seq.total_result_count

    @property
    def bills(self):
        '''
        A continuous iterator over the bills in the pages of results.
        Subsequent pages are loaded lazily, as the iterator "touches"
        them for the first time.
        '''
        for page in self.page_seq.pages:
            yield from page.results


class PageSequence(object):
    '''
    A sequence of pages of results, displayed on BillSearchResults.aspx,
    parsed into structured data.  
    
    For example, if there are 100 results, BillSearchResults.aspx displays 
    25 results on a page, so an instance of PageSequence would represent
    all four pages of results, available through the `PageSequence.pages`
    property.
    '''    
    def __init__(self, http_get, first_page):
        self._http_get = http_get

        ## lazily load subsequent pages
        self._pages = [ first_page, ]
        self._pages_lock = threading.Condition()

    # TODO: Make this class iterable and get rid of this property.
    @property
    def pages(self):
        page_index = 0
        while page_index < len(self._pages):
            yield self._pages[page_index]
            page_index += 1
        
        while not self._all_pages_loaded():
            self._ensure_page_loaded(page_index)
            yield self._pages[page_index]
            page_index += 1

    @property
    def total_result_count(self):
        return self._pages[0].total_result_count

    def _all_pages_loaded(self):
        return self._pages[-1].next_page_uri is None

    def _ensure_page_loaded(self, page_index):
        if len(self._pages) > page_index:
            return

        with self._pages_lock:
            while page_index >= len(self._pages) and not self._all_pages_loaded():
                last_loaded_page = self._pages[-1]
                uri = last_loaded_page.next_page_uri
                
                if DEBUG:
                    print('fetching {}...'.format(uri))

                response_body = self._http_get(uri)
                self._pages.append(Page(response_body, uri))


class Page(object):
    '''
    A single page of results, displayed on BillSearchResults.aspx,
    parsed into structured data.
    '''
    RESULT_COUNT_PATTERN = re.compile(
        "Bills [\d,]+ through [\d,]+ out of ([\d,]+) matches.", 
        re.IGNORECASE)

    @staticmethod
    def _parse_total_result_count(soup):
        '''
        Each total result count looks like
        
         <span id="lblMatches" style="display:inline-block;">
             Bills 1 through 25 out of 1,140 matches.
         </span>
        '''
        lbl_str = soup.find(id='lblMatches').string

        if lbl_str is None:
            return 0

        m = Page.RESULT_COUNT_PATTERN.search(lbl_str)
        return int(m.group(1).replace(',', ''))

    @staticmethod
    def _parse_results(soup, absolute_uri):
        '''
        Each result on this page begins with
        
         <table width="95%">
           <tbody>
             <tr width="100%">
               <td width="15%" nowrap="">
                   <a href="#" id="86R-HB 21" ...>
                       <img src="../Images/txicon.gif" ...>
        '''
        return [ 
            Result(_nearest_ancestor_table(txicon), absolute_uri)
            for txicon 
            in soup.find_all(name='img', attrs={'src':'../Images/txicon.gif'})
            ]

    @staticmethod
    def _parse_next_page_uri(soup, absolute_uri):
        '''
        Each "Next Page" link looks like

            <a href="BillSearchResults.aspx?CP=2&...">
                <img valign="bottom" 
                     src="../Images/icon_next_active.gif" 
                     alt="Navigate to next page">
            </a>
        '''
        img = soup.find(name='img', attrs={'alt':'Navigate to next page'})
        
        if img is None:
            return None

        a = img.parent
        relative_uri = a.attrs['href'] # e.g., "BillSearchResults.aspx?CP=3&..."
        return urllib.parse.urljoin(absolute_uri, relative_uri)

    def __init__(self, page_text, absolute_uri):
        soup = _parse(page_text)

        self.absolute_uri = absolute_uri
        self.next_page_uri = Page._parse_next_page_uri(soup, absolute_uri)
        self.total_result_count = Page._parse_total_result_count(soup)
        self.results = Page._parse_results(soup, absolute_uri)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.next_page_uri == other.next_page_uri \
                and self.total_result_count == other.total_result_count \
                and self.results == other.results
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    FORMAT = '''
        Page
            next_page_uri: {next_page_uri}
            total_result_count: {total_result_count}
            results: {results}
        '''

    def __str__(self):
        return Page.FORMAT.format(**self.__dict__)

    def __repr__(self):
        return self.__str__()


class Result(object):
    '''
    An individual result, displayed on BillSearchResults.aspx,
    parsed into structured data.  An example of the markup follows.

        <table width="95%">
            <tbody>
                <tr width="100%">
                    <td width="15%" nowrap="">
                        <a href="#" id="86R-HB 21" onclick="SetBillID(this.id); return dropdownmenu(this, event, menu)"
                            onmouseout="delayhidemenu()">
                            <img src="../Images/txicon.gif" class="noPrint" alt="Click for options">
                        </a>
                        <a href="../BillLookup/History.aspx?LegSess=86R&amp;Bill=HB21" target="_new">
                            HB 21
                        </a>
                    </td>
                    <td width="55%" valign="bottom" nowrap="" align="left"><strong>Author</strong>:
                        Canales
                    </td>
                    <td width="35%" valign="bottom" nowrap="" align="left"></td>
                </tr>
                <tr>
                    <td width="130" height="12"><strong>Last Action</strong>:&nbsp;</td>
                    <td colspan="2" height="12"><em>11/12/2018 H Filed</em></td>
                </tr>
                <tr>
                    <td width="130" nowrap=""><strong>Caption Version</strong>:</td>
                    <td colspan="2">Introduced</td>
                </tr>
                <tr>
                    <td width="130" valign="top"><strong>Caption</strong>:</td>
                    <td colspan="2">Relating to exempting textbooks purchased, used, or consumed by university and college
                        students from the sales and use tax for limited periods.</td>
                </tr>
            </tbody>
        </table>    
    '''

    '''
    A colon followed by whitespace and the author(s) name(s).

    Example:  ":\n\t Romero, Jr. | et al.\n\t"
    '''
    _AUTHOR_PATTERN = re.compile('\s*:\s*(.*)\s*') 

    @staticmethod
    def _parse_author(tds):
        m = Result._AUTHOR_PATTERN.match(tds[1].contents[1])
        return m.group(1).strip()

    @staticmethod
    def _parse_bill_link(tds, absolute_uri):
        bill_link = tds[0].contents[3]
        title = bill_link.string.strip()
        relative_history_uri = bill_link.attrs['href']
        abs_history_uri = urllib.parse.urljoin(absolute_uri, relative_history_uri)
        return title, abs_history_uri

    '''
    The date of the last action and the action taken.

    Example: "01/24/2019 H Filed"
    '''
    _LAST_ACTION_PATTERN = re.compile('\s*(\d+/\d+/\d+)(.*)\s*')

    @staticmethod
    def _parse_last_action(tds):
        action_and_date_str = ''.join(s for s in tds[4].strings)
        m = Result._LAST_ACTION_PATTERN.match(action_and_date_str)
        action = m.group(2).strip()
        date_str = m.group(1)
        date = datetime.datetime.strptime(date_str, '%m/%d/%Y').date()
        return action, date

    def __init__(self, table, absolute_uri):
        self.table = table
        tds = [ td for td in table.find_all('td') ]

        self.author = Result._parse_author(tds)
        self.caption_version = tds[6].string.strip()
        self.caption = tds[8].string.strip()
        self.title, self.history_uri = Result._parse_bill_link(tds, absolute_uri)
        self.last_action, self.last_action_date = Result._parse_last_action(tds)

        ## The following URIs are constructed by Javascript in the page, not
        ## included directly.  They'll be more brittle than the other attributes,
        ## so better to omit 'em 'til they're really needed.
        ## TODO: actions URI (e.g., "/BillLookup/Actions.aspx?LegSess=86R&Bill=HB%2021")
        ## TODO: text URI (e.g., "/BillLookup/Text.aspx?LegSess=86R&Bill=HB 21")

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.history_uri == other.history_uri
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return '{title} by {author} {last_action} {last_action_date}'.format(**self.__dict__)

    def __repr__(self):
        return self.__str__()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="txbillsearch",
    version="0.1.1",
    author="Ed Vinyard",
    author_email="ed.vinyard@gmail.com",
    description="Scrape Texas Legislature Online Bill Search results.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/EdVinyard/TxBillSearch",
    packages=setuptools.find_packages(),
    install_requires=[
        # KLUDGE: Keep this in sync with Pipfile manually for now!
        # See https://github.com/pypa/pipenv/issues/1263#issuecomment-362600555
        # for an alternate approach.
        'requests',
        'bs4',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
